from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import random
from datetime import timedelta


class ChromeSettings:

    def __init__(self):
        
        """Settings of the chrome browser."""
        

        # Create Chromeoptions instance
        self.options = webdriver.ChromeOptions()

        # Adding argument to disable the AutomationControlled flag
        self.options.add_argument("--disable-blink-features=AutomationControlled")
        self.options.add_argument("--headless")
        self.options.add_argument("--window-size=1920,1080")
        self.options.add_argument(
    "user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"
)


        # Disabling password manager pop up
        prefs = {
            "credentials_enable_service": False,
            "profile.password_manager_enabled": False,
        }
        self.options.add_experimental_option("prefs", prefs)

        # Exclude the collection of enable-automation switches
        self.options.add_experimental_option("excludeSwitches", ["enable-automation"])

        # Turn-off userAutomationExtension
        self.options.add_experimental_option("useAutomationExtension", False)
        self.options.add_experimental_option("detach", True)

        # Setting the driver path and requesting a page
        self.driver = webdriver.Chrome(options=self.options)

        # Changing the property of the navigator value for webdriver to undefined
        self.driver.execute_script(
            "Object.defineProperty(navigator, 'webdriver', {get: () => undefined})"
        )
        self.driver.set_window_size(1920, 1080)
    
        self.random_time = random.randrange(2, 11)
        self.loop_duration_time = timedelta(minutes=random.choice(range(15,30)))
        self.wait = WebDriverWait(self.driver, 10)



from .inputs import Inputs
from .chromesettings import ChromeSettings
from .firstwebpage import FirstWebPage
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import NoSuchElementException
import time 
import random



class AutoAssistant():
    
    
    def __init__(self, chromesettings:ChromeSettings,inputs_instance:Inputs,firstwp:FirstWebPage):
        
        self.chromesettings = chromesettings
        self.inputs_instance = inputs_instance
        self.firstwp = firstwp
        self.job_spec_name = None
        self.location_spec_name = None
        self.job_location_text = None
        self.job_title_text = None
        self.job_card_bodies = None
        self.job_card_body = None
        self.jobbody_amount = True
       
       



    def job_selection(self):
        
        """Finds the jobs from the website."""
        
        
        what_jobprocess = self.chromesettings.driver.find_element(By.CSS_SELECTOR, 'input[data-qa="searchKeywordInput"]')
        where_jobprocess = self.chromesettings.driver.find_element(By.CSS_SELECTOR, 'input[data-qa="searchLocationInput"]')
        
        what_value = what_jobprocess.get_attribute("value")
        where_value = where_jobprocess.get_attribute("value")
        

        self.job_spec_name = what_value
        self.location_spec_name = where_value
            
                 
        print(f'what_value={what_value}, where_value={where_value}')
        print(f'what_from_page={self.job_spec_name}, where_from_page={ self.location_spec_name}')
        
        time.sleep(10)
        self.job_card_bodies = self.chromesettings.driver.find_elements(By.CSS_SELECTOR, "div.job-card_jobCard__body__86jgk.card-body")
        
        while self.jobbody_amount:

            if len(self.job_card_bodies) < 3:
                self.jobbody_amount = False
                self.chromesettings.driver.close()
            
            else:
                for self.job_card_body in self.job_card_bodies:
                    
                    
                    job_location = self.job_card_body.find_element(By.CSS_SELECTOR, "li[data-qa='job-card-location']")
                    job_title = self.job_card_body.find_element(By.CSS_SELECTOR, "[data-qa='job-card-title']")
            
                    self.job_location_text = job_location.text
                    self.job_title_text = job_title.text
                    
                    
                    print(
                                f"Current value in the ITERATION = {self.job_spec_name} and  {self.location_spec_name} \nWeb element names {self.job_title_text} and {self.job_location_text}"
                            )
                
                    if  self.job_spec_name.lower() in self.job_title_text.lower() and self.location_spec_name.lower() in self.job_location_text.lower():
                        print(f'JOB NAME SELECTED: {self.job_spec_name} \nLOCATION NAME SELECTED: {self.location_spec_name}')
                        

                        try:
                            index_job_card = self.job_card_bodies.index(self.job_card_body)
                            print(f'this is the current amount of card bodies: {len(self.job_card_bodies)}')
                            
                            self.chromesettings.driver.execute_script("arguments[0].scrollIntoView(true);", self.job_card_body)

                            try:
                                main_job_card = self.job_card_body.find_element(By.CSS_SELECTOR, 'button[data-qa="applyJobBtn"]')
                            
                            except Exception as e:
                                print(f'No main job card {e}\n Trying to find the job suggestion')
                                try:
                                    unshortlist = self.job_card_body.find_element(By.CSS_SELECTOR, "button[aria-label='Unshortlist job']")   
                                    print(f"unshort list button already been clicked.")
                                except NoSuchElementException:      
                                    job_suggestion = self.job_card_body.find_element(By.CSS_SELECTOR, "button.job-card_btnShortlistJob__jgO8k.btn.btn-inline")
                                    
                                    self.chromesettings.driver.execute_script("arguments[0].click();", job_suggestion)
                                    self.job_card_bodies.pop(index_job_card)
                                    print('Your job suggestion found')

                            main_jobcard_active = self.job_card_body.find_element(By.CSS_SELECTOR, "button.job-card_applyBtn__2N2jy.btn.btn-secondary:not(.disabled)")
                            time.sleep(self.chromesettings.random_time)
                            self.chromesettings.driver.execute_script("arguments[0].click();", main_jobcard_active)
                            print("MAIN ACTIVE CLICKED")
                            
                            job_description = self.chromesettings.wait.until(EC.element_to_be_clickable((By.CSS_SELECTOR, "button[data-qa='submit-application-btn']")))
                            time.sleep(self.chromesettings.random_time)
                            job_description.click()
                            
                            try:
                                ok_button = self.chromesettings.wait.until(EC.element_to_be_clickable((By.CSS_SELECTOR, 'button.application-confirmation-modal_continueButton__i73Dl.btn.btn-primary')))
                                time.sleep(self.chromesettings.random_time)
                                ok_button.click()
                            
                            except Exception as e:
                                print(f'OK button exception: {e}')
                                x_button = self.chromesettings.wait.until(EC.element_to_be_clickable((By.CSS_SELECTOR, 'div > div.modal.overflow-auto.fade.show.d-block > div > div > header > button')))
                                x_button.click()

                                self.job_card_bodies.pop(index_job_card)
                                print(f'index of the job card {self.job_card_bodies.index(self.job_card_body)} successfully removed')
                        except Exception as e:
                                print(f'Main job card NOT FOUND: {e}')
                
                # Else condition
                try:
                    
                    next_page_button = self.chromesettings.driver.find_element(By.CSS_SELECTOR, "a.page-link.next[aria-label='Next page']")
                    self.chromesettings.driver.execute_script("arguments[0].click();", next_page_button)
                    
                    return self.job_selection()
                
                except Exception as e:
                    print(f'Next page button exception: {e}')
                    # return False

            


from .inputs import Inputs
from .chromesettings import ChromeSettings
from .firstwebpage import FirstWebPage
from .login import Login
from .reedautoassistant import AutoAssistant
import schedule
import time
import random



class MainInteraction():
    
    def __init__(self):
        self.input_instance = Inputs()
        
      
    def job_tasks(self):
        
        """Function that contain most of the interactions with the website"""
        
    
        chrosettings = ChromeSettings()
        login = Login(chrosettings,self.input_instance)
        firstwp = FirstWebPage(chrosettings,self.input_instance)
        autoassistant = AutoAssistant(chrosettings,self.input_instance,firstwp)
        
        try:
            login.loginpage_interaction()
            firstwp.webpage_interaction_firstwp()
            autoassistant.job_selection()
        except Exception as err:
            print(f"Error at {err}")
            chrosettings.driver.close()
            
    
        
maininteraction_instance = MainInteraction()
    

def main():
    """Scheduling of the job searching"""


    schedule.every(random.choice(range(17,22))).minutes.do(maininteraction_instance.job_tasks)
    while True:
        schedule.run_pending()
        time.sleep(1)
     
     
def run_as_script():
    
    """Runs the entire main.py file as a script"""
    
    maininteraction_instance.input_instance.input_collection()
    main()




if __name__ == "__main__":
    run_as_script()
 


from .chromesettings import ChromeSettings
from .inputs import Inputs
import random
import time
from selenium.webdriver.common.by import By





class FirstWebPage():
    
    def __init__(self,chromesettings:ChromeSettings,inputs_instance:Inputs):
        
        self.chromesettings = chromesettings
        self.inputs_instance = inputs_instance
        self.jobtitle_firstwp = None
        self.joblocation_firstwp = None
        self.what = None
        self.where = None
        
        
    def webpage_interaction_firstwp(self):
        
        
        """Interacts with the first webpage."""
        
        

        self.jobtitle_firstwp = random.choice(self.inputs_instance.jobtitle_list)
        self.what = self.chromesettings.driver.find_element(
            By.XPATH,
            "/html/body/div[1]/div/form/div[1]/div[1]/span/input",
        )
        time.sleep(self.chromesettings.random_time)
        self.what.send_keys(self.jobtitle_firstwp)

        self.joblocation_firstwp = random.choice(self.inputs_instance.joblocation_list)
        self.where = self.chromesettings.driver.find_element(
            By.XPATH,
            "/html/body/div[1]/div/form/div[1]/div[2]/span/input",
        )
        time.sleep(self.chromesettings.random_time)
        self.where.send_keys(self.joblocation_firstwp)
        

        search_jobs = self.chromesettings.driver.find_element(By.CSS_SELECTOR,"button.btn.btn-primary.btn-search")
        time.sleep(self.chromesettings.random_time)
        search_jobs.click()
        
        self.chromesettings.driver.refresh()
        
   
        time.sleep(10)
        last_week = self.chromesettings.driver.find_element(By.XPATH,"/html/body/div[1]/div[4]/div/div[3]/aside/div[2]/div/div[6]/div[2]/div/select/option[4]")
        last_week.click()
        
        
    
  

        



from .inputs import Inputs
from .chromesettings import ChromeSettings
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
import time
import random

class Login():
    
            
    def __init__(self, chromesettings:ChromeSettings,inputs_instance:Inputs):
        
        self.chromesettings = chromesettings
        self.inputs_instance = inputs_instance
        
            
    def loginpage_interaction(self):
        
        """Opens the sign up page and sends credentials."""
        
        self.chromesettings.driver.get("https://secure.reed.co.uk/login?state=hKFo2SB6a0ludTZFdW8zNmRNUXR6bWJsUFFfeEg3bTdQaW1ZOKFupWxvZ2luo3RpZNkgazc3dE9sbktIb3Y2N1IyeDNuekZ3WTBPa2xwTXQ3c3ijY2lk2SBUS2JBVXhQRUFEWEFYZGYyN05tWUp2MEtnNmFEZnJkdA&client=TKbAUxPEADXAXdf27NmYJv0Kg6aDfrdt&protocol=oauth2&scope=openid%20profile%20email%20offline_access&redirect_uri=https%3A%2F%2Fwww.reed.co.uk%2Fauthentication%2Flogin%2Fcallback&audience=https%3A%2F%2Fwww.reed.co.uk%2F&response_type=code&response_mode=query&nonce=a2pJRGxmYTVSdFA4MGNCNlFlMFhxVjY0MWNMQXFYTkkyMnRmaHpOTkotZw%3D%3D&code_challenge=iwsfbRafIlFXcvUcGPk_sNu65V148bDShu1QCMC4SKM&code_challenge_method=S256&auth0Client=eyJuYW1lIjoiYXV0aDAtc3BhLWpzIiwidmVyc2lvbiI6IjIuMS4zIn0%3D")

        
        time.sleep(10)
        email_bar = self.chromesettings.driver.find_element(By.ID, "signin_email")
        time.sleep(self.chromesettings.random_time)
        email_bar.send_keys(self.inputs_instance.email + Keys.ENTER)


        password_bar = self.chromesettings.driver.find_element(By.ID, "signin_password")
        time.sleep(self.chromesettings.random_time)
        password_bar.send_keys(self.inputs_instance.password)


        continue_button = self.chromesettings.driver.find_element(By.ID, "signin_button")
        time.sleep(self.chromesettings.random_time)
        continue_button.click()
        
         
        time.sleep(20)
        accept_button = self.chromesettings.driver.find_element(
        By.XPATH,"/html/body/div[2]/div[2]/div/div/div[2]/div/div/button"
        )
        accept_button.click()
        
        
    


import random

class Inputs:
    
    """Collects the data essential for the website interraction"""
    
    
    def __init__(self):
        
        self.email = None
        self.password = None
        self.job_title = None
        self.job_location = None 
        self.jobtitle_list = []
        self.joblocation_list = []
        
        
    def input_collection(self):
        
        """Collects data recquired for the program to work."""
       

        self.email = input("Please enter your Reed email: ")
        self.password = input("Please enter your Reed password: ")
        
  
        print("Up to 5 desired job titles and locations.\nIt is HIGHLY RECOMMENDED to be specific with the job title ex:'IT Support Specialist'")
            
        self.job_title = input("Enter desired job title: ")
        self.jobtitle_list.append(self.job_title)
        
        while True:
            
            job_titles_amount = input("Would you like to add an additional job title? Yes/No: ")
            
            if job_titles_amount == "Yes":
                self.job_title = input("Enter desired job title: ")
                self.jobtitle_list.append(self.job_title)
            elif job_titles_amount == "No":
                break
            else:
                print("Invalid input")
            
      
            
        self.job_location = input("Enter desired job location: ")
        self.joblocation_list.append(self.job_location)
        
        
        while True:
            
            job_locations_amount = input("Would you like to add an additional location? Yes/No: ")
            
            if job_locations_amount == "Yes":
                self.job_location = input("Enter desired job location: ")
                self.joblocation_list.append(self.job_location)
            elif job_locations_amount == "No":
                break
            else:
                print("Invalid input")
        
    
        print (f"job title list {self.jobtitle_list}\nJob location list {self.joblocation_list}")
        


           
        
    
    
    
    


from setuptools import setup, find_packages

setup(
    name="reedautomated",
    version="0.4.0",  
     packages=find_packages(include=['reedautomated', 'reedautomated.*']),
     install_requires=[
        "selenium",
        "schedule",
        "chromedriver"
    ],
      entry_points={
        "console_scripts": [
        "run-reedautoassistant = reedautomated.main:run_as_script"
        ]
    },
)
